/*
 * UriResolver.java
 *
 * Created on April 17, 2007, 6:04 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package test.converter;

import com.sun.ws.rest.api.WebApplicationException;
import java.lang.reflect.Method;
import java.net.HttpURLConnection;
import java.net.URI;
import java.net.URL;
import javax.xml.bind.JAXBContext;
import test.service.PersistenceService;

/**
 * Utility class for resolving an uri into an entity.
 * 
 * @author lukas
 */
public class UriResolver {
    
    private static ThreadLocal<UriResolver> instance = new ThreadLocal<UriResolver>() {
        protected UriResolver initialValue() {
            return new UriResolver();
        }
    };
    
    private boolean inProgress = false;
    
    private UriResolver() {
    }
    
    /**
     * Returns an instance of UriResolver.
     *
     * @return an instance of UriResolver.
     */
    public static UriResolver getInstance() {
        return instance.get();
    }
    
    private static void removeInstance() {
        instance.remove();
    }
    
    /**
     * Returns the entity associated with the given uri.
     *
     * @param type the converter class used to unmarshal the entity from XML
     * @param uri the uri identifying the entity
     * @return the entity associated with the given uri
     */
    public <T> T resolve(Class<T> type, URI uri) {
        if (inProgress) return null;
        
        inProgress = true;
        
        try {
            URL url = uri.toURL();
            HttpURLConnection conn = (HttpURLConnection)url.openConnection();
            conn.setRequestMethod("GET");
            
            if (conn.getResponseCode() == 200) {
                JAXBContext context = JAXBContext.newInstance(type);
                Object obj = context.createUnmarshaller().unmarshal(conn.getInputStream());
                resolveEntity(obj);
                
                return (T) obj;
            }
            
            return null;
        } catch (Exception ex) {
            throw new WebApplicationException(ex);
        } finally {
            removeInstance();
        }
    }
    
    private void resolveEntity(Object obj) {
        try {
            Method method = obj.getClass().getMethod("getEntity");
            Object entity = method.invoke(obj);
            entity = PersistenceService.getInstance().resolveEntity(entity);
            method = obj.getClass().getMethod("setEntity", Object.class);
            method.invoke(obj, entity);
        } catch (Exception ex) {
        }
    }
}
