/*
 * MicroMarketResource.java
 * 
 * Created on Apr 17, 2007, 6:04:41 PM
 * 
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package test.service;

import com.sun.ws.rest.api.Entity;
import com.sun.ws.rest.api.UriTemplate;
import com.sun.ws.rest.api.UriParam;
import com.sun.ws.rest.api.HttpMethod;
import com.sun.ws.rest.api.ProduceMime;
import com.sun.ws.rest.api.ConsumeMime;
import com.sun.ws.rest.api.core.HttpContext;
import com.sun.ws.rest.api.response.HttpResponse;
import java.net.URI;
import javax.annotation.Resource;
import com.sun.ws.rest.api.NotFoundException;
import javax.persistence.NoResultException;
import test.MicroMarket;
import test.converter.MicroMarketConverter;

/**
 *
 * @author lukas
 */
@UriTemplate("{zipCode}/")
public class MicroMarketResource {
    @Resource
    private HttpContext context;

    public MicroMarketResource() {
    }

    /**
     * Constructor used for instantiating an instance of dynamic resource.
     *
     * @param context HttpContext inherited from the parent resource
     */

    public MicroMarketResource(HttpContext context) {
        this.context = context;
    }

    /**
     * Get method for retrieving an instance of MicroMarket identified by id in XML format.
     *
     * @param id identifier for the entity
     * @return an instance of MicroMarketConverter
     */

    @HttpMethod("GET")
    @ProduceMime("application/xml")
    public MicroMarketConverter getXML(@UriParam("zipCode")
                                       String id) {
        try {
            return new MicroMarketConverter(getEntity(id), getUri());
        }
        finally {
            PersistenceService.getInstance().close();
        }
    }

    /**
     * Put method for updating an instance of MicroMarket identified by id using XML as the input format.
     *
     * @param id identifier for the entity
     * @param data an MicroMarketConverter entity that is deserialized from a XML stream
     * @return a HttpResponse
     */

    @HttpMethod("PUT")
    @ConsumeMime("application/xml")
    public HttpResponse putXML(@UriParam("zipCode")
                               String id,
                               Entity<MicroMarketConverter> data) {
        PersistenceService service = PersistenceService.getInstance();

        try {
            service.beginTx();
            updateEntity(getEntity(id), data.getContent().getEntity());
            service.commitTx();
            return new HttpResponse();
        }
        finally {
            service.close();
        }
    }

    /**
     * Delete method for deleting an instance of MicroMarket identified by id.
     *
     * @param id identifier for the entity
     * @return a HttpResponse
     */

    @HttpMethod("DELETE")
    public HttpResponse delete(@UriParam("zipCode")
                               String id) {
        PersistenceService service = PersistenceService.getInstance();

        try {
            service.beginTx();
            MicroMarket entity = getEntity(id);

            service.removeEntity(entity);
            service.commitTx();
            return new HttpResponse();
        }
        finally {
            service.close();
        }
    }

    /**
     * Returns an instance of MicroMarket identified by id.
     *
     * @param id identifier for the entity
     * @return an instance of MicroMarket
     */

    protected MicroMarket getEntity(String id) {
        try {
            return (MicroMarket) PersistenceService.getInstance().createNamedQuery("MicroMarket.findByZipCode").setParameter("zipCode",
                                                                                                                             id).getSingleResult();
        }
        catch (NoResultException ex) {
            throw new NotFoundException("Resource for " + getUri() +
                                        " does not exist.");
        }
    }

    /**
     * Updates entity using data from newEntity.
     *
     * @param entity the entity to update
     * @param newEntity the entity containing the new data
     * @return the updated entity
     */

    protected MicroMarket updateEntity(MicroMarket entity, MicroMarket newEntity) {
        newEntity.setZipCode(entity.getZipCode());
        entity = PersistenceService.getInstance().mergeEntity(newEntity);
        return entity;
    }

    /**
     * Returns the URI associated with this resource.
     *
     * @return URI associated with this resource
     */

    private URI getUri() {
        return context.getHttpRequestContext().getURI();
    }

}
