/*
 * MicroMarketsResource.java
 * 
 * Created on Apr 17, 2007, 6:04:41 PM
 * 
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package test.service;

import com.sun.ws.rest.api.Entity;
import com.sun.ws.rest.api.UriTemplate;
import com.sun.ws.rest.api.HttpMethod;
import com.sun.ws.rest.api.ProduceMime;
import com.sun.ws.rest.api.SubResources;
import com.sun.ws.rest.api.core.HttpContext;
import com.sun.ws.rest.api.response.Created;
import com.sun.ws.rest.api.response.HttpResponse;
import java.net.URI;
import java.util.Collection;
import javax.annotation.Resource;
import test.MicroMarket;
import test.converter.MicroMarketsConverter;
import test.converter.MicroMarketConverter;

/**
 *
 * @author lukas
 */
@UriTemplate("/microMarkets/")
@SubResources({MicroMarketResource.class})
public class MicroMarketsResource {
    @Resource
    private HttpContext context;

    public MicroMarketsResource() {
    }

    /**
     * Constructor used for instantiating an instance of dynamic resource.
     *
     * @param context HttpContext inherited from the parent resource
     */

    public MicroMarketsResource(HttpContext context) {
        this.context = context;
    }

    /**
     * Get method for retrieving a collection of MicroMarket instance in XML format.
     *
     * @return an instance of MicroMarketsConverter
     */

    @HttpMethod("GET")
    @ProduceMime("application/xml")
    public MicroMarketsConverter getXML() {
        try {
            return new MicroMarketsConverter(getEntities(), getUri());
        }
        finally {
            PersistenceService.getInstance().close();
        }
    }

    /**
     * Post method for creating an instance of MicroMarket using XML as the input format.
     *
     * @param data an MicroMarketsConverter entity that is deserialized from an XML stream
     * @return an instance of MicroMarketsConverter
     */

    @HttpMethod("POST")
    @ProduceMime("application/xml")
    public HttpResponse postXML(Entity<MicroMarketConverter> data) {
        PersistenceService service = PersistenceService.getInstance();

        try {
            service.beginTx();
            MicroMarket entity = data.getContent().getEntity();

            createEntity(entity);
            service.commitTx();
            return new Created(getUri().resolve(entity.getZipCode() + "/"));
        }
        finally {
            service.close();
        }
    }

    /**
     * Returns all the entities associated with this resource.
     *
     * @return a collection of MicroMarket instances
     */

    protected Collection<MicroMarket> getEntities() {
        return PersistenceService.getInstance().createQuery("SELECT e FROM MicroMarket e").getResultList();
    }

    /**
     * Persist the given entity.
     *
     * @param entity the entity to persist
     */

    protected void createEntity(MicroMarket entity) {
        PersistenceService.getInstance().persistEntity(entity);
    }

    /**
     * Returns the URI associated with this resource.
     *
     * @return URI associated with this resource
     */

    private URI getUri() {
        return context.getHttpRequestContext().getURI();
    }

}
